//SETUP
var PROTO_PATH = __dirname + '/point.proto';
var assert = require('assert');
var async = require('async');
const axios = require('axios');
var _ = require('lodash');
var grpc = require('@grpc/grpc-js');
var protoLoader = require('@grpc/proto-loader');
var packageDefinition = protoLoader.loadSync(
    PROTO_PATH,
    {keepCase: true,
     longs: String,
     enums: String,
     defaults: true,
     oneofs: true
    });
var protoDescriptor = grpc.loadPackageDefinition(packageDefinition);
var pointPackage = protoDescriptor.pointPackage;
/**
 * @param {!Object} call
 * @param {function():?} callback
 */

 var count = 0;




//SERVICE
function sendPoint(call, callback) {
  axios.get('http://localhost:8090').then(resp=>{
    callback(null, resp["data"]["point"]);
  })
}










/**
 * @param {!Object} call
 */
function sendPointStream(call) {
  
  function sender(point) {
    return (callback) => {
      call.write(point);
      _.delay(callback, 1); // in ms
    };
  }

  var senders = [];
  var count = 10;
  for (var i = 0; i < count; i++) {
    senders[i] = sender(generatePoint());
  }
  async.series(senders, () => {
    call.end();
  });
}



//Generate randomPoint
function generatePoint(){
  var xVal = Math.floor(Math.random()*20);
  var yVal = Math.floor(Math.random()*20);
  count++;
  //console.log("Generated point. ID: " + count + "  x: " + xVal + " y: " + yVal);
  
  return {id:count, x:xVal,y:yVal};
}



/**
 * @return {!Object} gRPC server
 */
function getServer() {
  var server = new grpc.Server();
  server.addService(pointPackage.Point.service, {
    readPoint: sendPoint,
    readPointsStream: sendPointStream,
  });
  return server;
}

if (require.main === module) {
  var server = getServer();
  server.bindAsync(
    '0.0.0.0:9090', grpc.ServerCredentials.createInsecure(), (err, port) => {
      assert.ifError(err);
      server.start();
      console.log("connected");
  });
}
exports.getServer = getServer;
